@@ -1,8 +1,10 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
+ * Copyright (C) 2003-2004 Imendio HB
  * Copyright (C) 2003 CodeFactory AB
  * Copyright (C) 2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2003 Mikael Hallendal <micke@imendio.com>
+ * Copyright (C) 2003 Alvaro del Castillo <acs@barrapunto.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,8 +32,10 @@
 #include <libgnomeui/gnome-entry.h>
 #include <glade/glade.h>
 #include <gtk/gtk.h>
-#include <libpq-fe.h>
-#include "planner-main-window.h"
+#include <gconf/gconf-client.h>
+#include <libgda/libgda.h>
+#include "planner-window.h"
+#include "planner-application.h"
 #include "planner-plugin.h"
 
 #define d(x)
@@ -43,18 +47,21 @@
 #define PASSWORD   "sql-plugin-password"
 #define PROJECT_ID "sql-plugin-project-id"
 
+#define GCONF_PATH "/apps/planner/plugins/sql"
+
+#define STOP_ON_ERR GDA_COMMAND_OPTION_STOP_ON_ERRORS
 
 typedef struct {
 	GtkWidget *open_dialog;
 } SQLPluginPriv; 
 
-static gint     sql_plugin_retrieve_project_id (MgPlugin           *plugin,
+static gint     sql_plugin_retrieve_project_id (PlannerPlugin      *plugin,
 						gchar              *server,
 						gchar              *port,
 						gchar              *database,
 						gchar              *login,
 						gchar              *password);
-static gboolean sql_plugin_retrieve_db_values  (MgPlugin           *plugin,
+static gboolean sql_plugin_retrieve_db_values  (PlannerPlugin      *plugin,
 						const gchar        *title,
 						gchar             **server,
 						gchar             **port,
@@ -67,15 +74,20 @@ static void     sql_plugin_open                (BonoboUIComponent  *component,
 static void     sql_plugin_save                (BonoboUIComponent  *component,
 						gpointer            user_data,
 						const gchar        *cname);
-void            plugin_init                    (MgPlugin           *plugin,
-						MgMainWindow       *main_window);
+static GdaDataModel * 
+                sql_execute_query              (GdaConnection      *con, 
+					        gchar              *query);
+void            plugin_init                    (PlannerPlugin      *plugin,
+						PlannerWindow      *main_window);
 void            plugin_exit                    (void);
 
 
 
 enum {
 	COL_ID,
-	COL_NAME
+	COL_NAME,
+	COL_PHASE,
+	COL_REVISION
 };
 
 static BonoboUIVerb verbs[] = {
@@ -84,34 +96,68 @@ static BonoboUIVerb verbs[] = {
 	BONOBO_UI_VERB_END
 };
 
+
+/**
+ * Helper to execute a SQL query using GDA
+ */
+static GdaDataModel *
+sql_execute_query (GdaConnection *con, gchar *query)
+{
+	GdaCommand   *cmd;
+	GdaDataModel *res;
+
+	cmd = gda_command_new (query, GDA_COMMAND_TYPE_SQL, STOP_ON_ERR);
+	res = gda_connection_execute_single_command  (con, cmd, NULL);
+	gda_command_free (cmd);
+	return res;
+}
+
+
 /**
  * Helper to get an int.
  */
 static gint
-get_int (PGresult *res, gint i, gint j)
+get_int (GdaDataModel *res, gint row, gint column)
 {
-	const gchar *str;
+	gchar    *str;
+	GdaValue *value;
+	gint      i;
+
+	value = (GdaValue *) gda_data_model_get_value_at (res, column, row);
+	if (value == NULL) {
+		g_warning ("Failed to get a value: (%d,%d)", column, row);
+		return INT_MAX;
+	}
 	
-	str = PQgetvalue (res, i, j);
-	return strtol (str, NULL, 10);
+	str = gda_value_stringify (value);
+	i = strtol (str, NULL, 10);
+	g_free (str);
+
+	return i;
 }
 
 /**
  * Helper to get an UTF-8 string.
  */
 static gchar *
-get_string (PGresult *res, gint i, gint j)
+get_string (GdaDataModel *res, gint row, gint column)
 {
-	const gchar *str;
-	gchar *ret;
-	gsize len;
+	gchar    *str;
+	gchar    *ret;
+	gsize     len;
+	GdaValue *value;
 	
-	str = PQgetvalue (res, i, j);
-
+	value = (GdaValue *) gda_data_model_get_value_at (res, column, row);
+	if (value == NULL) {
+		g_warning ("Failed to get a value: (%d,%d)", column, row);
+		return "";
+	}
+	
+	str = gda_value_stringify (value);
 	len = strlen (str);
 	
 	if (g_utf8_validate (str, len, NULL)) {
-		return g_strdup (str);
+		return str;
 	}
 
 	/* First, try to convert to UTF-8 from the current locale. */
@@ -126,6 +172,8 @@ get_string (PGresult *res, gint i, gint j)
 		/* Give up. */
 		ret = g_strdup (_("Invalid Unicode"));
 	}
+
+	g_free (str);
 	
 	return ret;
 }
@@ -203,8 +251,8 @@ create_sql_uri (const gchar *server,
 }
 
 static void
-show_error_dialog (MgPlugin    *plugin,
-		   const gchar *str)
+show_error_dialog (PlannerPlugin *plugin,
+		   const gchar   *str)
 {
 	GtkWindow *window;
 	GtkWidget *dialog;
@@ -234,22 +282,30 @@ selection_changed_cb (GtkTreeSelection *selection, GtkWidget *ok_button)
 	gtk_widget_set_sensitive (ok_button, sensitive);
 }
 
+static void
+row_activated_cb (GtkWidget         *tree_view,
+		  GtkTreePath       *path,
+		  GtkTreeViewColumn *column,
+		  GtkWidget         *ok_button)
+{
+	gtk_widget_activate (ok_button);
+}
+
 /**
  * Display a list with projects and let the user select one. Returns the project
  * id of the selected one.
  */
 static gint
-sql_plugin_retrieve_project_id (MgPlugin *plugin,
-				gchar    *server,
-				gchar    *port,
-				gchar    *database,
-				gchar    *login,
-				gchar    *password)
+sql_plugin_retrieve_project_id (PlannerPlugin *plugin,
+				gchar         *server,
+				gchar         *port,
+				gchar         *database,
+				gchar         *login,
+				gchar         *password)
 {
-	PGconn            *conn;
- 	gchar             *pgoptions = NULL;
-	gchar             *pgtty = NULL;
-	PGresult          *res;
+	GdaConnection     *conn;
+	GdaDataModel      *res;
+	GdaClient         *client;
 	gchar             *str;
 	GladeXML          *gui;
 	GtkWidget         *dialog;
@@ -263,44 +319,51 @@ sql_plugin_retrieve_project_id (MgPlugin *plugin,
 	gint               project_id;
 	GtkTreeSelection  *selection;
 	GtkTreeIter        iter;
+	gchar             *db_txt;
+	const gchar       *dsn_name = "planner-auto";
+	const gchar       *provider = "PostgreSQL";
 
-	conn = PQsetdbLogin (server, port, pgoptions, pgtty, database, login, password);
- 	if (PQstatus (conn) == CONNECTION_BAD) {
-		str = g_strdup_printf (_("Connection to database '%s' failed:\n%s"),
-				       database,
-				       PQerrorMessage(conn));
+	db_txt = g_strdup_printf ("DATABASE=%s",database);
+	gda_config_save_data_source (dsn_name, 
+                                     provider, 
+                                     db_txt,
+                                     "planner project", login, password);
+	g_free (db_txt);
+
+	client = gda_client_new ();
+	
+	conn = gda_client_open_connection (client, dsn_name, NULL, NULL, 0);
+
+	if (!GDA_IS_CONNECTION (conn)) {
+		str = g_strdup_printf (_("Connection to database '%s' failed."), database);
 		show_error_dialog (plugin, str);
-		g_free (str);		
-
+		g_free (str);
 		return -1;
 	}
 
-	res = PQexec (conn, "BEGIN");
-	if (!res || PQresultStatus (res) != PGRES_COMMAND_OK) {
+	res = sql_execute_query (conn, "BEGIN");
+	if (res == NULL) {
 		g_warning ("BEGIN command failed.");
-
-		PQclear (res);
-		PQfinish (conn);
 		return -1;
 	}
-	PQclear (res);
+	g_object_unref (res);
+	res = NULL;
 
-	res = PQexec (conn,
-		      "DECLARE mycursor CURSOR FOR SELECT proj_id, name FROM project"); 
-	if (!res || PQresultStatus (res) != PGRES_COMMAND_OK) {
+	res = sql_execute_query (conn,
+				 "DECLARE mycursor CURSOR FOR SELECT proj_id, name,"
+				 "phase, revision FROM project ORDER by proj_id ASC");
+
+	if (res == NULL) {
 		g_warning ("DECLARE CURSOR command failed (project).");
-
-		PQclear (res);
-		PQfinish (conn);
 		return -1;
 	}
+	g_object_unref (res);
+	
 
-	res = PQexec (conn, "FETCH ALL in mycursor");
-	if (!res || PQresultStatus (res) != PGRES_TUPLES_OK) {
+	res = sql_execute_query (conn, "FETCH ALL in mycursor");
+
+	if (res == NULL) {
 		g_warning ("FETCH ALL failed.");
-		
-		PQclear (res);
-		PQfinish (conn);
 		return -1;
 	}
 
@@ -311,50 +374,95 @@ sql_plugin_retrieve_project_id (MgPlugin *plugin,
 
 	g_object_unref (gui);
 	
-	liststore = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
+	liststore = gtk_list_store_new (4, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (liststore));
 
+	cell = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (_("ID"),
+							cell,
+							"text", COL_ID,
+							NULL);
+	gtk_tree_view_column_set_resizable (col, TRUE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
+	
 	cell = gtk_cell_renderer_text_new ();
 	col = gtk_tree_view_column_new_with_attributes (_("Project"),
 							cell,
 							"text", COL_NAME,
 							NULL);
+	gtk_tree_view_column_set_resizable (col, TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
+	
+	cell = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (_("Phase"),
+							cell,
+							"text", COL_PHASE,
+							NULL);
+	gtk_tree_view_column_set_resizable (col, TRUE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
+	
+	cell = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (_("Revision"),
+							cell,
+							"text", COL_REVISION,
+							NULL);
+	gtk_tree_view_column_set_resizable (col, TRUE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
+	
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (treeview));
+	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE); 
 	
-	g_signal_connect (selection, "changed",
+	g_signal_connect (selection,
+			  "changed",
 			  G_CALLBACK (selection_changed_cb),
 			  ok_button);
+
+	g_signal_connect (treeview,
+			  "row_activated",
+			  G_CALLBACK (row_activated_cb),
+			  ok_button);
 	
-	for (i = 0; i < PQntuples (res); i++) {
+	for (i = 0; i < gda_data_model_get_n_rows (res); i++) {
 		gint   id;
 		gchar *name;
+		gchar *phase;
+		gint   revision;
 		
 		id = get_int (res, i, 0);
 		name = get_string (res, i, 1);
+		phase = get_string (res, i, 2);
+		revision = get_int (res, i, 3);
 
+		/* FIXME: needs fixing in the database backend. */
+		if (strcmp (phase, "NULL") == 0) {
+			g_free (phase);
+			phase = g_strdup ("");
+		}
+		
 		gtk_list_store_append (GTK_LIST_STORE (liststore), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (liststore), 
 				    &iter,
 				    COL_ID, id,
 				    COL_NAME, name,
+				    COL_PHASE, phase,
+			            COL_REVISION, revision,
 				    -1);
 
 		g_free (name);
+		g_free (phase);
 	}
 
-	if (PQntuples (res) == 0) {
+	if (gda_data_model_get_n_columns (res) == 0) {
 		gtk_widget_set_sensitive (ok_button, FALSE);
 	}
 	
-	PQclear (res);
+	g_object_unref (res);
+
+	res = sql_execute_query (conn,"CLOSE mycursor");
 	
-	res = PQexec (conn, "CLOSE mycursor");
-	PQclear (res);
-
-	PQfinish (conn);
-
+	g_object_unref (res);
 	
 	gtk_widget_show_all (dialog);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -384,33 +492,35 @@ sql_plugin_retrieve_project_id (MgPlugin *plugin,
 }
 
 static gboolean
-sql_plugin_retrieve_db_values (MgPlugin     *plugin,
-			       const gchar  *title,
-			       gchar       **server,
-			       gchar       **port,
-			       gchar       **database,
-			       gchar       **login,
-			       gchar       **password)
+sql_plugin_retrieve_db_values (PlannerPlugin  *plugin,
+			       const gchar    *title,
+			       gchar         **server,
+			       gchar         **port,
+			       gchar         **database,
+			       gchar         **login,
+			       gchar         **password)
 {
-	GladeXML    *gui;
-	GtkWidget   *dialog;
-	gchar       *str;
-	gint         response;
-	GtkWidget   *server_entry;
-	GtkWidget   *db_entry;
-	GtkWidget   *user_entry;
-	GtkWidget   *password_entry;
-	gboolean     ret;
+	GladeXML           *gui;
+	GtkWidget          *dialog;
+	gchar              *str;
+	gint                response;
+	GtkWidget          *server_entry;
+	GtkWidget          *db_entry;
+	GtkWidget          *user_entry;
+	GtkWidget          *password_entry;
+	gboolean            ret;
+	PlannerApplication *application;
+	GConfClient        *gconf_client;
+
+	application = planner_window_get_application (plugin->main_window);
+	
+	gconf_client = planner_application_get_gconf_client ();
 
 	gui = glade_xml_new (GLADEDIR"/sql.glade", "open_dialog" , NULL);
 	dialog = glade_xml_get_widget (gui, "open_dialog");
 
 	gtk_window_set_title (GTK_WINDOW (dialog), title);
 
-	str = g_strdup_printf ("<b>%s</b>", title);
-	gtk_label_set_markup (GTK_LABEL (glade_xml_get_widget (gui, "title_label")), str);
-	g_free (str);
-	
 	server_entry = gnome_entry_gtk_entry (
 		GNOME_ENTRY (glade_xml_get_widget (gui, "server_entry")));
 	db_entry = gnome_entry_gtk_entry (
@@ -419,6 +529,24 @@ sql_plugin_retrieve_db_values (MgPlugin     *plugin,
 		GNOME_ENTRY (glade_xml_get_widget (gui, "user_entry")));
 	password_entry = glade_xml_get_widget (gui, "password_entry");
 
+	str = gconf_client_get_string (gconf_client, GCONF_PATH "/server", NULL);
+	if (str) {
+		gtk_entry_set_text (GTK_ENTRY (server_entry), str);
+		g_free (str);
+	}
+
+	str = gconf_client_get_string (gconf_client, GCONF_PATH "/database", NULL);
+	if (str) {
+		gtk_entry_set_text (GTK_ENTRY (db_entry), str);
+		g_free (str);
+	}
+
+	str = gconf_client_get_string (gconf_client, GCONF_PATH "/username", NULL);
+	if (str) {
+		gtk_entry_set_text (GTK_ENTRY (user_entry), str);
+		g_free (str);
+	}
+	
 	g_object_unref (gui);
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -431,6 +559,16 @@ sql_plugin_retrieve_db_values (MgPlugin     *plugin,
 		*login = strdup_null_if_empty (gtk_entry_get_text (GTK_ENTRY (user_entry)));
 		*password = strdup_null_if_empty (gtk_entry_get_text (GTK_ENTRY (password_entry)));
 
+		gconf_client_set_string (gconf_client,
+					 GCONF_PATH "/server", *server ? *server : "",
+					 NULL);
+		gconf_client_set_string (gconf_client,
+					 GCONF_PATH "/database", *database ? *database : "",
+					 NULL);
+		gconf_client_set_string (gconf_client,
+					 GCONF_PATH "/username", *login ? *login : "",
+					 NULL);
+
 		ret = TRUE;
 		break;
 
@@ -449,18 +587,18 @@ sql_plugin_open (BonoboUIComponent *component,
 		 gpointer           user_data,
 		 const gchar       *cname)
 {
-	MgPlugin      *plugin = user_data;
-	MgApplication *application;
-	GtkWidget     *window;
-	MrpProject    *project;
-	gchar         *server = NULL;
-	gchar         *port = NULL;
-	gchar         *database = NULL;
-	gchar         *login = NULL;
-	gchar         *password = NULL;
-	gint           project_id = -1;
-	gchar         *uri = NULL;
-	GError        *error = NULL;
+	PlannerPlugin      *plugin = user_data;
+	PlannerApplication *application;
+	GtkWidget          *window;
+	MrpProject         *project;
+	gchar              *server = NULL;
+	gchar              *port = NULL;
+	gchar              *database = NULL;
+	gchar              *login = NULL;
+	gchar              *password = NULL;
+	gint                project_id = -1;
+	gchar              *uri = NULL;
+	GError             *error = NULL;
 
 	if (!sql_plugin_retrieve_db_values (plugin,
 					    _("Open from Database"),
@@ -488,9 +626,9 @@ sql_plugin_open (BonoboUIComponent *component,
 
 	uri = create_sql_uri (server, port, database, login, password, project_id);
 
-	project = planner_main_window_get_project (plugin->main_window);
+	project = planner_window_get_project (plugin->main_window);
 	window = GTK_WIDGET (plugin->main_window);
-	application = planner_main_window_get_application (plugin->main_window);
+	application = planner_window_get_application (plugin->main_window);
 	
 	if (mrp_project_is_empty (project)) {
 		GObject *object = G_OBJECT (window);
@@ -514,7 +652,7 @@ sql_plugin_open (BonoboUIComponent *component,
 		GObject *object;
 		
 		window = planner_application_new_window (application);
-		project = planner_main_window_get_project (MG_MAIN_WINDOW (window));
+		project = planner_window_get_project (PLANNER_WINDOW (window));
 		
 		object = G_OBJECT (window);
 		
@@ -556,18 +694,18 @@ sql_plugin_save (BonoboUIComponent *component,
 		 gpointer           user_data,
 		 const gchar       *cname)
 {
-	MgPlugin   *plugin = user_data;
-	MrpProject *project;
-	GObject    *object;
-	gchar      *server = NULL;
-	gchar      *port = NULL;
-	gchar      *database = NULL;
-	gchar      *login = NULL;
-	gchar      *password = NULL;
-	gchar      *uri = NULL;
-	GError     *error = NULL;
+	PlannerPlugin  *plugin = user_data;
+	MrpProject    *project;
+	GObject       *object;
+	gchar         *server = NULL;
+	gchar         *port = NULL;
+	gchar         *database = NULL;
+	gchar         *login = NULL;
+	gchar         *password = NULL;
+	gchar         *uri = NULL;
+	GError        *error = NULL;
 		
-	project = planner_main_window_get_project (plugin->main_window);
+	project = planner_window_get_project (plugin->main_window);
 
 	if (!sql_plugin_retrieve_db_values (plugin,
 					    _("Save to Database"),
@@ -617,8 +755,8 @@ plugin_exit (void)
 }
 
 G_MODULE_EXPORT void 
-plugin_init (MgPlugin     *plugin,
-	     MgMainWindow *main_window)
+plugin_init (PlannerPlugin *plugin,
+	     PlannerWindow *main_window)
 {
 	BonoboUIContainer *ui_container;
 	BonoboUIComponent *ui_component;
@@ -638,7 +776,7 @@ plugin_init (MgPlugin     *plugin,
 			   "sql-plugin",
 			   plugin);
 	
-	ui_container = planner_main_window_get_ui_container (main_window);
+	ui_container = planner_window_get_ui_container (main_window);
 	ui_component = bonobo_ui_component_new_default ();
 	
 	bonobo_ui_component_set_container (ui_component, 
